#!/usr/bin/env python

"""Growl 0.6+ Network Protocol Client for Python suitable for use by a shell
script.

This script is a modification of the netgrowl.py script found at 
the.taoofmac.com. I removed the unit test in the if __name__ == '__main__'
block. I added options so this could be run from a shell script easily.

I'm releasing my changes under the same license that Rui used, to keep things
simple.

The original script informaiton is found below:

Growl 0.6 Network Protocol Client for Python
__version__ = "0.6" # will always match Growl version
__author__ = "Rui Carmo (http://the.taoofmac.com)"
__copyright__ = "(C) 2004 Rui Carmo. Code under BSD License."
__contributors__ = "Ingmar J Stein (Growl Team)"
"""

__version__ = "0.6" # will always match Growl version
__author__ = "Zach White (http://darkstar.frop.org)"
__copyright__ = "(C) 2007 Rui Carmo and Zach White. Code under BSD License."
__contributors__ = "Ingmar J Stein (Growl Team)"

import struct
import md5
from socket import AF_INET, SOCK_DGRAM, socket

GROWL_UDP_PORT=9887
GROWL_PROTOCOL_VERSION=1
GROWL_TYPE_REGISTRATION=0
GROWL_TYPE_NOTIFICATION=1

class GrowlRegistrationPacket:
	"""Builds a Growl Network Registration packet.
		 Defaults to emulating the command-line growlnotify utility."""

	def __init__(self, application="growlnotify", password = None ):
		self.notifications = []
		self.defaults = [] # array of indexes into notifications
		self.application = application.encode("utf-8")
		self.password = password
	# end def


	def addNotification(self, notification="Command-Line Growl Notification", enabled=True):
		"""Adds a notification type and sets whether it is enabled on the GUI"""
		self.notifications.append(notification)
		if enabled:
			self.defaults.append(len(self.notifications)-1)
	# end def


	def payload(self):
		"""Returns the packet payload."""
		self.data = struct.pack( "!BBH",
														 GROWL_PROTOCOL_VERSION,
														 GROWL_TYPE_REGISTRATION,
														 len(self.application) )
		self.data += struct.pack( "BB",
															len(self.notifications),
															len(self.defaults) )
		self.data += self.application
		for notification in self.notifications:
			encoded = notification.encode("utf-8")
			self.data += struct.pack("!H", len(encoded))
			self.data += encoded
		for default in self.defaults:
			self.data += struct.pack("B", default)
		self.checksum = md5.new()
		self.checksum.update(self.data)
		if self.password:
			 self.checksum.update(self.password)
		self.data += self.checksum.digest()
		return self.data
	# end def
# end class


class GrowlNotificationPacket:
	"""Builds a Growl Network Notification packet.
		 Defaults to emulating the command-line growlnotify utility."""

	def __init__(self, application="growlnotify",
							 notification="Command-Line Growl Notification", title="Title",
							 description="Description", priority = 0, sticky = False, password = None ):
		self.application	= application.encode("utf-8")
		self.notification = notification.encode("utf-8")
		self.title				= title.encode("utf-8")
		self.description	= description.encode("utf-8")
		flags = (priority & 0x07) * 2
		if priority < 0:
			flags |= 0x08
		if sticky:
			flags = flags | 0x0001
		self.data = struct.pack( "!BBHHHHH",
														 GROWL_PROTOCOL_VERSION,
														 GROWL_TYPE_NOTIFICATION,
														 flags,
														 len(self.notification),
														 len(self.title),
														 len(self.description),
														 len(self.application) )
		self.data += self.notification
		self.data += self.title
		self.data += self.description
		self.data += self.application
		self.checksum = md5.new()
		self.checksum.update(self.data)
		if password:
			 self.checksum.update(password)
		self.data += self.checksum.digest()
	# end def

	def payload(self):
		"""Returns the packet payload."""
		return self.data
	# end def
# end class


if __name__ == '__main__':
	import getopt, sys
	application = 'Demo App'
	port = GROWL_UDP_PORT
	host = 'localhost'
	password = 'password'
	title = 'Test'
	level = 'debug'
	sendRegistration = False

	options = 'a:p:h:P:t:l:s'
	longoptions = ['application=', 'port=', 'host=', 'password=', 'title=', 'level=', 'send-registration' ]
	try:
		opts, args = getopt.getopt(sys.argv[1:], options, longoptions)
	except getopt.GetoptError:
		print 'Invalid option.'
		sys.exit(2)
	### Process our options
	for o, a in opts:
		if o in ("-a", "--application"):
			application = a
		elif o in ("-p", "--port"):
			port = int(a)
		elif o in ("-h", "--host"):
			host = a
		elif o in ("-P", "--password"):
			password = a
		elif o in ("-t", "--title"):
			title = a
		elif o in ("-l", "--level"):
			level = a
		elif o in ("-s", "--send-registration"):
			sendRegistration = True
	message = ' '.join(args)

	addr = (host, port)
	s = socket(AF_INET,SOCK_DGRAM)
	if sendRegistration:
		p = GrowlRegistrationPacket(application=application, password=password)
		levels = ['emerg', 'alert', 'crit', 'err', 'warning', 'notice', 'info', 'debug']
		for level in levels:
			if level in ['debug', 'info']:
				p.addNotification(level, enabled=False)
			else:
				p.addNotification(level, enabled=True)
		s.sendto(p.payload(), addr)
	else:
		p = GrowlNotificationPacket(application=application,
			notification=level, title=title,
			description=message,
			priority=1, sticky=True, password=password
		)
		s.sendto(p.payload(), addr)

		s.close()
